@@ -1,9 +1,11 @@
 """Common functions and variables for all commands
 """
 
+from __future__ import unicode_literals
 __copyright__ = """
 Copyright (C) 2007-2010 Novell Inc.
-Author: Alex Tsariounov <alext@novell.com>
+Copyright (C) 2013-2018 SUSE
+Author: Alex Tsariounov <tsariounov@gmail.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
