@@ -1,9 +1,13 @@
 """Process manipulation command
 """
 
+from __future__ import unicode_literals
+from builtins import str
+from future.utils import lrange
 __copyright__ = """
 Copyright (C) 2007-2010 Novell Inc.
-Author: Alex Tsariounov <alext@novell.com>
+Copyright (C) 2013-2018 SUSE
+Author: Alex Tsariounov <tsariounov@gmail.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
@@ -19,7 +23,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 """
 
-import sys, os, re, logging, pwd, grp
+import sys, os, io, re, logging, pwd, grp
 from optparse import OptionParser, make_option
 
 from cpuset import config
@@ -321,7 +325,7 @@ def list_sets(args):
     l = []
     if isinstance(args, list):
         for s in args: 
-            if isinstance(s, str):
+            if isstr(s):
                 l.extend(cset.find_sets(s))
             elif not isinstance(s, cset.CpuSet):
                 raise CpusetException(
@@ -330,7 +334,7 @@ def list_sets(args):
             else:
                 l.append(s)
     else:
-        if isinstance(args, str):
+        if isstr(args):
             l.extend(cset.find_sets(args))
         elif not isinstance(args, cset.CpuSet):
             raise CpusetException(
@@ -351,14 +355,14 @@ def list_sets(args):
 def move(fromset, toset, plist=None, verb=None, force=None):
     log.debug('entering move, fromset=%s toset=%s list=%s force=%s verb=%s', 
               fromset, toset, plist, force, verb)
-    if isinstance(fromset, str):
+    if isstr(fromset):
         fset = cset.unique_set(fromset)
     elif not isinstance(fromset, cset.CpuSet) and plist == None:
         raise CpusetException(
                 "passed fromset=%s, which is not a string or CpuSet" % fromset)
     else:
         fset = fromset
-    if isinstance(toset, str):
+    if isstr(toset):
         tset = cset.unique_set(toset)
     elif not isinstance(toset, cset.CpuSet):
         raise CpusetException(
@@ -524,7 +528,7 @@ def selective_move(fset, tset, plist=None, kthread=None, force=None, threads=Non
     move(None, target, tasks)
 
 def run(tset, args, usr_par=None, grp_par=None):
-    if isinstance(tset, str):
+    if isstr(tset):
         s = cset.unique_set(tset)
     elif not isinstance(tset, cset.CpuSet):
         raise CpusetException(
@@ -589,11 +593,11 @@ def pidspec_to_list(pidspec, fset=None, threads=False):
     log.debug('entering pidspecToList, pidspec=%s fset=%s threads=%s', 
               pidspec, fset, threads)
     if fset: 
-        if isinstance(fset, str): fset = cset.unique_set(fset)
+        if isstr(fset): fset = cset.unique_set(fset)
         elif not isinstance(fset, cset.CpuSet):
             raise CpusetException("passed fset=%s, which is not a string or CpuSet" % fset)
         log.debug('from-set specified as: %s', fset.path)
-    if not isinstance(pidspec, str):
+    if not isstr(pidspec):
         raise CpusetException('pidspec=%s is not a string' % pidspec)
     groups = pidspec.split(',')
     plist = []
@@ -620,7 +624,7 @@ def pidspec_to_list(pidspec, fset=None, threads=False):
                 log.debug(' added single pid: %s', items[0])
         elif len(items) == 2:
             # a range of pids, only include those that exist
-            rng = [str(x) for x in range(int(items[0]), int(items[1])+1) 
+            rng = [str(x) for x in lrange(int(items[0]), int(items[1])+1)
                            if os.access('/proc/'+str(x), os.F_OK)]
             if fset:
                 for tsk in rng:
@@ -654,7 +658,7 @@ def pidspec_to_list(pidspec, fset=None, threads=False):
         log.debug('thread matching activated, looking for threads...')
         dups = 0
         hits = 0
-        for task in pdict.keys():
+        for task in list(pdict.keys()):
             dirs = os.listdir('/proc/'+str(task)+'/task')
             if len(dirs) > 1:
                 hits += 1
@@ -664,7 +668,7 @@ def pidspec_to_list(pidspec, fset=None, threads=False):
                         continue
                     pdict[thread] = True
         log.debug('found %s multithreaded containers and %s duplicates', hits, dups)
-    plist = pdict.keys()
+    plist = list(pdict.keys())
     log.debug('returning parsed pid list of %s tasks: %s', len(plist), plist)
     return plist
 
@@ -684,7 +688,7 @@ def move_pidspec(pidspec, toset, fset=None, threads=False):
 
 def task_detail(pid, width=70):
     # scheduler policy definitions
-    policy = ['o', 'f', 'r', 'b']
+    policy = ['o', 'f', 'r', 'b', '?', 'i', 'd']
     # stat location definitions
     statdef = {
         'pid': 0,
@@ -703,16 +707,21 @@ def task_detail(pid, width=70):
     pid = str(pid)
     if not os.access('/proc/'+pid, os.F_OK):
         raise CpusetException('task "%s" does not exist' % pid)
-    status = file('/proc/'+pid+'/status', 'r').readlines()
+    status = io.open('/proc/'+pid+'/status', 'r',encoding="iso8859-1").readlines()
     stdict = {}
     for line in status:
         try:
             stdict[line.split()[0][:-1]] = line.split(':')[1].strip()
         except:
             pass  # sometimes, we get an extra \n out of this file...
-    stat = file('/proc/'+pid+'/stat', 'r').readline()
-    stat = stat.split()
-    cmdline = file('/proc/'+pid+'/cmdline').readline()
+    stat = io.open('/proc/'+pid+'/stat', 'r',encoding="iso8859-1").readline()
+    # we assume parentheses appear only around the name
+    stat_right_paren = stat.rfind(')')
+    stat_left_paren = stat.find('(')
+    stat = [stat[:stat_left_paren-1]] + \
+           [stat[stat_left_paren:stat_right_paren+1]] + \
+           stat[stat_right_paren+2:].split()
+    cmdline = io.open('/proc/'+pid+'/cmdline',encoding="iso8859-1").readline()
     # assume that a zero delimits the cmdline (it does now...)
     cmdline = cmdline.replace('\0', ' ')
 
@@ -727,11 +736,17 @@ def task_detail(pid, width=70):
 
     out2 = []
     out2.append(stdict['State'].split()[0])
-    out2.append(policy[int(stat[statdef['rtpolicy']])])
+    policy_code=int(stat[statdef['rtpolicy']])
+    out2.append(policy[policy_code] if policy_code<len(policy) else '?')
     if stat[statdef['rtpolicy']] == '0':
         out2.append('th')
     elif stat[statdef['rtpolicy']] == '3':
         out2.append('at')
+    # SCHED_ISO is reserved but not yet implemented as of Linux v4.4
+    elif stat[statdef['rtpolicy']] == '4':
+        out2.append('??')
+    elif stat[statdef['rtpolicy']] == '5':
+        out2.append('dl')
     else:
         if int(stat[statdef['rtpriority']]) < 10:
             out2.append('_')
