@@ -1,9 +1,12 @@
 """Cpuset manipulation command
 """
 
+from __future__ import unicode_literals
+from builtins import str
 __copyright__ = """
 Copyright (C) 2007-2010 Novell Inc.
-Author: Alex Tsariounov <alext@novell.com>
+Copyright (C) 2013-2018 SUSE
+Author: Alex Tsariounov <tsariounov@gmail.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
@@ -286,7 +289,7 @@ def destroy_sets(sets, recurse=False, force=False):
 def destroy(name):
     """destroy one cpuset by name as cset or string"""
     log.debug('entering destroy, name=%s', name)
-    if isinstance(name, str):
+    if isstr(name):
         set = cset.unique_set(name)
     elif not isinstance(name, cset.CpuSet):
         raise CpusetException(
@@ -400,7 +403,7 @@ def modify(name, cpuspec=None, memspec=None, cx=None, mx=None):
     """modify one cpuset by name, cpuspec, memspec, cpu and mem exclusive flags"""
     log.debug('entering modify, name=%s cpuspec=%s memspec=%s cx=%s mx=%s',
               name, cpuspec, memspec, cx, mx)
-    if isinstance(name, str):
+    if isstr(name):
         nset = cset.unique_set(name)
     elif not isinstance(name, cset.CpuSet):
         raise CpusetException(
@@ -416,7 +419,7 @@ def modify(name, cpuspec=None, memspec=None, cx=None, mx=None):
 def active(name):
     """check that cpuset by name or cset is ready to be used"""
     log.debug("entering active, name=%s", name)
-    if isinstance(name, str):
+    if isstr(name):
         set = cset.unique_set(name)
     elif not isinstance(name, cset.CpuSet):
         raise CpusetException("passing bogus name=%s" % name)
@@ -440,7 +443,7 @@ def set_header(indent=None):
 def set_details(name, indent=None, width=None, usehex=False):
     """return string of cpuset details"""
     if width == None: width = 0
-    if isinstance(name, str):
+    if isstr(name):
         set = cset.unique_set(name)
     elif not isinstance(name, cset.CpuSet):
         raise CpusetException("passing bogus set=%s" % name)
@@ -480,10 +483,10 @@ def set_details(name, indent=None, width=None, usehex=False):
 
     if width != 0 and len(tst) > width:
         target = width - len(out)
-        patha = set.path[:len(set.path)/2-3]
-        pathb = set.path[len(set.path)/2:]
-        patha = patha[:target/2-3]
-        pathb = pathb[-target/2:]
+        patha = set.path[:len(set.path)//2-3]
+        pathb = set.path[len(set.path//2):]
+        patha = patha[:target//2-3]
+        pathb = pathb[-target//2:]
         out += patha + '...' + pathb
     else:
         out = tst
