@@ -1,9 +1,12 @@
 """Shield supercommand
 """
 
+from __future__ import unicode_literals
+from builtins import str
 __copyright__ = """
 Copyright (C) 2007-2010 Novell Inc.
-Author: Alex Tsariounov <alext@novell.com>
+Copyright (C) 2013-2018 SUSE
+Author: Alex Tsariounov <tsariounov@gmail.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
@@ -298,7 +301,7 @@ def func(parser, options, args):
             else:
                 try:
                     proc.move_pidspec(options.pid, to_set, from_set, options.threads)
-                except CpusetException, err:
+                except CpusetException as err:
                     if str(err).find('do not match all criteria') != -1:
                         log.info("--> hint: perhaps use --force if sure of command")
                         raise
@@ -375,7 +378,7 @@ def make_shield(cpuspec, kthread):
         try:
             set.create(USR_SET, cpuspec, memspec, True, False)
             set.create(SYS_SET, cpuspec_inv, memspec, True, False)
-        except Exception, instance:
+        except Exception as instance:
             # unroll
             try: set.destroy(USR_SET)
             except: pass
