@@ -1,9 +1,16 @@
 """Cpuset class and cpuset graph, importing module will create model
 """
+from __future__ import unicode_literals
+from __future__ import print_function
+from future.utils import lrange
 
+import io
+from builtins import str
+from builtins import object
 __copyright__ = """
 Copyright (C) 2007-2010 Novell Inc.
-Author: Alex Tsariounov <alext@novell.com>
+Copyright (C) 2013-2017 SUSE
+Author: Alex Tsariounov <tsariounov@gmail.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
@@ -36,6 +43,11 @@ class CpuSet(object):
     # a relative path from this basepath.
     sets = {}
     basepath = ''
+    cpus_path = '/cpus'
+    mems_path = '/mems'
+    cpu_exclusive_path = '/cpu_exclusive'
+    mem_exclusive_path = '/mem_exclusive'
+    tasks_path = '/tasks'
 
     def __init__(self, path=None):
         log.debug("initializing CpuSet")
@@ -56,20 +68,23 @@ class CpuSet(object):
                 del CpuSet.sets
                 CpuSet.sets = {}
             CpuSet.sets[self.path] = self
+
+            # if mounted as a cgroup controller, switch file name format
+            if not os.access(path + CpuSet.cpus_path, os.F_OK):
+                CpuSet.cpus_path = '/cpuset.cpus'
+                CpuSet.mems_path = '/cpuset.mems'
+                CpuSet.cpu_exclusive_path = '/cpuset.cpu_exclusive'
+                CpuSet.mem_exclusive_path = '/cpuset.mem_exclusive'
+
             # bottom-up search otherwise links will not exist
             log.debug("starting bottom-up discovery walk...")
             for dir, dirs, files in os.walk(path, topdown=False):
                 log.debug("*** walking %s", dir)
-                if dir != CpuSet.basepath:
-                    node = CpuSet(dir)
-                else:
-                    node = self
+                node = self if dir == CpuSet.basepath else CpuSet(dir)
                 node.subsets = []
                 for sub in dirs:
-                    if len(sub) > 0:
-                        relpath = os.path.join(dir,sub).replace(CpuSet.basepath, '')
-                    else:
-                        relpath = '/'
+                    relpath = (os.path.join(dir,sub).replace(CpuSet.basepath, '')
+                               if len(sub) > 0 else '/')
                     node.subsets.append(CpuSet.sets[relpath])
                 log.debug("%s has %i subsets: [%s]", dir, 
                           len(node.subsets), '|'.join(dirs))
@@ -85,7 +100,7 @@ class CpuSet(object):
                 else:
                     parpath = dir[0:dir.rfind('/')]
                     log.debug('parpath decodes to: %s from dir of: %s', parpath, dir)
-                    if CpuSet.sets.has_key(parpath):
+                    if parpath in CpuSet.sets:
                         log.debug("parent is %s", parpath)
                         node.parent = CpuSet.sets[parpath]
                     else:
@@ -95,16 +110,14 @@ class CpuSet(object):
         else:
             # one new cpuset node
             log.debug("new cpuset node absolute: %s", path)
-            if len(path) > len(CpuSet.basepath):
-                path = path.replace(CpuSet.basepath, '')
-            else:
-                path = '/'
+            path = (path.replace(CpuSet.basepath, '')
+                    if len(path) > len(CpuSet.basepath) else '/')
             log.debug(" relative: %s", path)
-            if CpuSet.sets.has_key(path):
+            if path in CpuSet.sets:
                 log.debug("the cpuset %s already exists, skipping", path)
                 self = CpuSet.sets[path]  # questionable....
                 return
-            cpus = CpuSet.basepath + path + "/cpus"
+            cpus = CpuSet.basepath + path +  CpuSet.cpus_path
             if not os.access(cpus, os.F_OK):
                 # not a cpuset directory
                 str = '%s is not a cpuset directory' % (CpuSet.basepath + path)
@@ -116,21 +129,14 @@ class CpuSet(object):
 
     def locate_cpusets(self):
         log.debug("locating cpuset filesystem...")
-        cpuset = re.compile(r"none (/.+) cpuset .+")
-        cgroup = re.compile(r"none (/.+) cgroup .+")
+        cpuset_mount_regex = re.compile(r"^[^ ]+ (/.+) (?:cpuset |cgroup (?:[^ ]*,)?cpuset[, ])")
         path = None
-        f = file("/proc/mounts")
+        f = io.open("/proc/mounts",encoding="iso8859-1")
         for line in f:
-            res = cpuset.search(line)
+            res = cpuset_mount_regex.search(line)
             if res:
                 path = res.group(1)
                 break
-            else:
-                if cgroup.search(line):
-                    groups = line.split()
-                    if re.search("cpuset", groups[3]):
-                        path = groups[1]
-                        break
         f.close()
 
         if not path:
@@ -153,71 +159,58 @@ class CpuSet(object):
         self.name = path[path.rfind('/')+1:]
         log.debug("...name=%s", self.name)
 
+    def read_first_line_from(self, file_to_read):
+        f = io.open(CpuSet.basepath+self.path+file_to_read, encoding="iso8859-1")
+        retval = f.readline().strip()
+        f.close()
+        return retval
+
+    def write_value_to(self, file_to_write, value):
+        log.debug("-> prop_set %s.%s = %s", self.path, file_to_write, value)
+        f = io.open(CpuSet.basepath+self.path+file_to_write, 'w', encoding="iso8859-1")
+        f.write(str(value))
+        f.close()
+
+    def write_01_to(self, file_to_write, value):
+        self.write_value_to(file_to_write, '1' if value else '0')
+
     # Properties of cpuset node
     def delprop(self):
-        raise AttributeError, "deletion of properties not allowed"
+        raise AttributeError("deletion of properties not allowed")
 
     def getcpus(self): 
-        f = file(CpuSet.basepath+self.path+"/cpus")
-        return f.readline()[:-1]
+        return self.read_first_line_from(CpuSet.cpus_path)
     def setcpus(self, newval):
         cpuspec_check(newval)
-        f = file(CpuSet.basepath+self.path+"/cpus",'w')
-        f.write(str(newval))
-        f.close()
-        log.debug("-> prop_set %s.cpus = %s", self.path, newval) 
+        self.write_value_to(CpuSet.cpus_path, newval)
     cpus = property(fget=getcpus, fset=setcpus, fdel=delprop, doc="CPU specifier")
 
     def getmems(self): 
-        f = file(CpuSet.basepath+self.path+"/mems")
-        return f.readline()[:-1]
+        return self.read_first_line_from(CpuSet.mems_path)
     def setmems(self, newval): 
         # FIXME: check format for correctness
-        f = file(CpuSet.basepath+self.path+"/mems",'w')
-        f.write(str(newval))
-        f.close()
-        log.debug("-> prop_set %s.mems = %s", self.path, newval) 
+        self.write_value_to(CpuSet.mems_path, newval)
     mems = property(getmems, setmems, delprop, "Mem node specifier")
     
     def getcpuxlsv(self): 
-        f = file(CpuSet.basepath+self.path+"/cpu_exclusive")
-        if f.readline()[:-1] == '1':
-            return True
-        else:
-            return False
+        return self.read_first_line_from(CpuSet.cpu_exclusive_path) == '1'
     def setcpuxlsv(self, newval):
-        log.debug("-> prop_set %s.cpu_exclusive = %s", self.path, newval) 
-        f = file(CpuSet.basepath+self.path+"/cpu_exclusive",'w')
-        if newval:
-            f.write('1')
-        else:
-            f.write('0')
-        f.close()
+        self.write_01_to(CpuSet.cpu_exclusive_path, newval)
     cpu_exclusive = property(getcpuxlsv, setcpuxlsv, delprop, 
                              "CPU exclusive flag")
 
     def getmemxlsv(self): 
-        f = file(CpuSet.basepath+self.path+"/mem_exclusive")
-        if f.readline()[:-1] == '1':
-            return True
-        else:
-            return False
+        return self.read_first_line_from(CpuSet.mem_exclusive_path) == '1'
     def setmemxlsv(self, newval):
-        log.debug("-> prop_set %s.mem_exclusive = %s", self.path, newval) 
-        f = file(CpuSet.basepath+self.path+"/mem_exclusive",'w')
-        if newval:
-            f.write('1')
-        else:
-            f.write('0')
-        f.close()
+        self.write_01_to(CpuSet.mem_exclusive_path, newval)
     mem_exclusive = property(getmemxlsv, setmemxlsv, delprop, 
                              "Memory exclusive flag")
 
     def gettasks(self):
-        f = file(CpuSet.basepath+self.path+"/tasks")
-        lst = []
-        for task in f: lst.append(task[:-1])
-        return lst
+        f = io.open(CpuSet.basepath+self.path+CpuSet.tasks_path,encoding="iso8859-1")
+        lst = map(lambda line: line.strip(), f.readlines())
+        f.close()
+        return list(lst)
     def settasks(self, tasklist):
         notfound = []
         unmovable = []
@@ -229,10 +222,10 @@ class CpuSet(object):
             prog = False
         for task in tasklist:
             try:
-                f = file(CpuSet.basepath+self.path+"/tasks",'w')
+                f = io.open(CpuSet.basepath+self.path+CpuSet.tasks_path,'w',encoding="iso8859-1")
                 f.write(task)
                 f.close()
-            except Exception, err:
+            except Exception as err:
                 if str(err).find('No such process') != -1:
                     notfound.append(task)
                 elif str(err).find('Invalid argument'):
@@ -241,7 +234,7 @@ class CpuSet(object):
                     raise
             if prog:
                 tick += 1
-                pb(tick)
+                pb.progress(tick)
         if len(notfound) > 0:
             log.info('**> %s tasks were not found, so were not moved', len(notfound))
             log.debug(' not found: %s', notfound)
@@ -261,7 +254,9 @@ def lookup_task_from_proc(pid):
     log.debug("entering lookup_task_from_proc, pid = %s", str(pid))
     path = "/proc/"+str(pid)+"/cpuset"
     if os.access(path, os.F_OK):
-        set = file(path).readline()[:-1]
+        f = io.open(path,encoding="iso8859-1")
+        set = f.readline().strip()
+        f.close()
         log.debug('lookup_task_from_proc: found task %s cpuset: %s', str(pid), set)
         return set
     # FIXME: add search for threads here...
@@ -389,8 +384,8 @@ def cpuspec_to_hex(cpuspec):
             number |= 1 << int(items[0])
         elif len(items) == 2: 
             il = [int(ii) for ii in items]
-            if il[1] >= il[0]: rng = range(il[0], il[1]+1)
-            else: rng = range(il[1], il[0]+1)
+            if il[1] >= il[0]: rng = lrange(il[0], il[1]+1)
+            else: rng = lrange(il[1], il[0]+1)
             log.debug(' group=%s has cpu range of %s', sub, rng)
             for num in rng: number |= 1 << num
         else:
@@ -412,7 +407,7 @@ def memspec_check(memspec):
 
 def cpuspec_inverse(cpuspec):
     """calculate inverse of cpu specification"""
-    cpus = [0 for x in range(maxcpu+1)]
+    cpus = [0 for x in lrange(maxcpu+1)]
     groups = cpuspec.split(',')
     log.debug("cpuspec_inverse(%s) maxcpu=%d groups=%d", 
               cpuspec, maxcpu, len(groups))
@@ -425,29 +420,23 @@ def cpuspec_inverse(cpuspec):
                 continue
             cpus[int(items[0])] = 1
         elif len(items) == 2:
-            for x in range(int(items[0]), int(items[1])+1):
+            for x in lrange(int(items[0]), int(items[1])+1):
                 cpus[x] = 1
         else:
             raise CpusetException("cpuspec(%s) has bad group %s" % (cpuspec, set))
     log.debug("cpuspec array: %s", cpus)
     # calculate inverse of array
-    for x in range(0, len(cpus)):
-        if cpus[x] == 0:
-            cpus[x] = 1
-        else:
-            cpus[x] = 0
+    for x in lrange(0, len(cpus)):
+        cpus[x] = int(cpus[x] == 0)
     log.debug("      inverse: %s", cpus)
     # build cpuspec expression
     nspec = ""
     ingrp = False
-    for x in range(0, len(cpus)):
+    for x in lrange(0, len(cpus)):
         if cpus[x] == 0 and ingrp:
             nspec += str(begin)
             if x > begin+1: 
-                if cpus[x] == 1:
-                    nspec += '-' + str(x)
-                else:
-                    nspec += '-' + str(x-1)
+                nspec += '-' + str(x if cpus[x] else x-1)
             ingrp = False
         if cpus[x] == 1:
             if not ingrp: 
@@ -491,26 +480,26 @@ if __name__ == '__main__':
     except:
         pass
 
-    print 'Max cpu on system:', maxcpu
-    print 'All cpu mask: 0x%s' % allcpumask
+    print('Max cpu on system:', maxcpu)
+    print('All cpu mask: 0x%s' % allcpumask)
 
-    print '------- find_sets tests --------'
-    print 'Find by root of "root" -> ', find_sets("root")
-    print 'Find by path of "/" -> ', find_sets("/")
+    print('------- find_sets tests --------')
+    print('Find by root of "root" -> ', find_sets("root"))
+    print('Find by path of "/" -> ', find_sets("/"))
 
-    print 'Find by path of "/csettest/one" -> ', find_sets("/csettest/one")
-    print 'Find by name of "one" -> ', find_sets("one")
-    print 'Find by path of "/csettest/two" -> ', find_sets("/csettest/two")
-    print 'Find by name of "two" -> ', find_sets("two")
+    print('Find by path of "/csettest/one" -> ', find_sets("/csettest/one"))
+    print('Find by name of "one" -> ', find_sets("one"))
+    print('Find by path of "/csettest/two" -> ', find_sets("/csettest/two"))
+    print('Find by name of "two" -> ', find_sets("two"))
 
-    print 'Find by path of "/csettest/one/x" -> ', find_sets("/csettest/one/x")
-    print 'Find by name of "x" -> ', find_sets("x")
-    print 'Find by path of "/csettest/two/y" -> ', find_sets("/csettest/two/y")
-    print 'Find by name of "y" -> ', find_sets("y")
+    print('Find by path of "/csettest/one/x" -> ', find_sets("/csettest/one/x"))
+    print('Find by name of "x" -> ', find_sets("x"))
+    print('Find by path of "/csettest/two/y" -> ', find_sets("/csettest/two/y"))
+    print('Find by name of "y" -> ', find_sets("y"))
 
     try:
         node = find_sets("cantfindmenoway")
-        print 'Found "cantfindmenoway??!? -> ', node
-    except CpusetException, err:
-        print 'Caught exeption for non-existant set (correctly)-> ', err
+        print('Found "cantfindmenoway??!? -> ', node)
+    except CpusetException as err:
+        print('Caught exeption for non-existant set (correctly)-> ', err)
 
