@@ -1,6 +1,9 @@
+from __future__ import unicode_literals
+
 __copyright__ = """
 Copyright (C) 2007-2010 Novell Inc.
-Author: Alex Tsariounov <alext@novell.com>
+Copyright (C) 2013-2018 SUSE
+Author: Alex Tsariounov <tsariounov@gmail.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License version 2 as
@@ -16,4 +19,4 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 """
 
-version = '1.5.6'
+version = '1.6'
